<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\ClientsRequest;
use App\Http\Requests\Admin\ProvidersRequest;
use App\Models\Category;
use App\Models\City;
use App\Models\Package;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;

class ProvidersController extends Controller
{

    public function index()
    {
        $rows = User::whereType('provider')->get();
        $packages = Package::all();

        return view('admin.providers.index', compact('rows', 'packages'));
    }

    public function contractPackage($provider_id, Request $request)
    {
        $this->validate($request, [
            'subscription_id' => 'required'
        ]);


        $package = Package::findOrFail($request['subscription_id']);
        $provider = User::findOrFail($provider_id);
        $provider->subscription_id = $request['subscription_id'];
        $provider->end_subscription = Carbon::now()->addDays($package->duration);
        $provider->update();
        toast('تم الاشتراك بنجاح', 'success');
        return redirect()->back();
    }

    public function renewPackage($provider_id)
    {
        $provider = User::findOrFail($provider_id);
        $package = Package::findOrFail($provider->subscription_id);
        $provider->end_subscription = Carbon::now()->addDays($package->duration);
        $provider->update();
        toast('تم تجديد الباقه بنجاح', 'success');
        return redirect()->back();
    }

    public function create()
    {
        $cities = City::all();
        $categories = Category::all();
        $subscriptions = Package::all();
        return view('admin.providers.create', compact('cities', 'subscriptions', 'categories'));
    }

    public function store(ProvidersRequest $request)
    {
        $provider = new User();
        $provider->email = $request['email'];
        $provider->type = 'provider';
        $provider->name = $request['name'];
        $provider->mobile = $request['mobile'];
        $provider->city_id = $request['city_id'];
        $provider->category_id = $request['category_id'];
        $provider->subscription_id = $request['subscription_id'];
        $provider->admin_accept = $request['admin_accept'];
        $package = Package::findOrFail($request['subscription_id']);
        $provider->end_subscription = Carbon::now()->addDays($package->duration);
        $provider->password = bcrypt($request['password']);
        $provider->status = $request['status'];
        if ($request->hasFile('profile_image')) {
            $filename = time() . '.' . $request['profile_image']->getClientOriginalExtension();
            $request['profile_image']->move(public_path('images/users'), $filename);
            $provider->profile_image = $filename;
        }
        $provider->save();
        toast('تمت العملية بنجاح', 'success');
        return redirect()->route('providers.index');
    }

    public function edit($id)
    {
        $row = User::findOrFail($id);
        $cities = City::all();
        $categories = Category::all();

        return view('admin.providers.edit', compact('row', 'cities', 'categories'));
    }

    public function update($id, Request $request)
    {
        $this->validate($request, [
            'email' => 'required|max:191|email|unique:users,email,' . $id,
            'name' => 'required|max:191',
            'password' => 'nullable|min:6|max:191',
            'category_id' => 'required',
            'mobile' => 'required|unique:users,mobile,' . $id,
            'city_id' => 'required'
        ]);
        $provider = User::findOrFail($id);
        $provider->email = $request['email'];
        $provider->type = 'provider';
        $provider->name = $request['name'];
        $provider->mobile = $request['mobile'];
        $provider->city_id = $request['city_id'];
        $provider->category_id = $request['category_id'];
        $provider->admin_accept = $request['admin_accept'];
        if ($request->password) $provider->password = bcrypt($request['password']);
        $provider->status = $request['status'];
        if ($request->hasFile('profile_image')) {
            $filename = time() . '.' . $request['profile_image']->getClientOriginalExtension();
            $request['profile_image']->move(public_path('images/users'), $filename);
            $provider->profile_image = $filename;
        }
        $provider->update();
        toast('تمت العملية بنجاح', 'success');
        return redirect()->route('providers.index');
    }

    public function destroy($id)
    {

        User::destroy($id);
        toast('تمت العملية بنجاح', 'success');
        return back();

    }
}
