<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\StoreServiceRequest;
use App\Http\Requests\Api\UpdateServiceRequest;
use App\Http\Resources\Api\ServicesResource;
use App\Models\Favourite;
use App\Models\Service;
use App\Models\ServiceImage;
use Carbon\Carbon;
use Illuminate\Http\Request;
use JWTAuth;

class ServicesController extends Controller
{
    public function storeService(StoreServiceRequest $request)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }

//        if (Carbon::now() > Carbon::parse($user->end_subscription)){
//            return  response()->json(['value'=>false,'msg'=>\Lang::get('data.subscriptionEnded')]);
//        }


        $service = new Service();
        $service['user_id'] = $user->id;
        $service['category_id'] = $request['category_id'];
        $service['city_id'] = $request['city_id'];
        $service['lat'] = $request['lat'];
        $service['lng'] = $request['lng'];
        $service['description'] = $request['description'];
        $service['provider_number'] = $request['provider_number'];
        $service['employee_number'] = $request['employee_number'];
        $service['price_from'] = $request['price_from'];
        $service['price_to'] = $request['price_to'];
        $service['title'] = $request['title'];
        $service->save();
        $i = 1;

        foreach ($request['images'] as $image) {

            $filename = $i . time() . '.' . $image->getClientOriginalExtension();
            $image->move(public_path('images/services'), $filename);

            $serviceImage = new ServiceImage();
            $serviceImage->service_id = $service->id;
            $serviceImage->image = $filename;
            $serviceImage->save();
            $i++;
        }


        return response()->json(['value' => true, 'data' => new ServicesResource($service)]);

    }

    public function updateService(UpdateServiceRequest $request)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }


        $service = Service::findOrFail($request['service_id']);
        $deleted_images = explode(',', $request['deleted_images']);
        if ($request['deleted_images'] != null) {
            if (count($deleted_images) == count($service->images)) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.cantDeleteAllImages')]);
            }
        }

        $service['user_id'] = $user->id;
        $service['category_id'] = $request['category_id'];
        $service['city_id'] = $request['city_id'];
        $service['lat'] = $request['lat'];
        $service['lng'] = $request['lng'];
        $service['description'] = $request['description'];
        $service['provider_number'] = $request['provider_number'];
        $service['employee_number'] = $request['employee_number'];
        $service['price_from'] = $request['price_from'];
        $service['price_to'] = $request['price_to'];
        $service['title'] = $request['title'];
        $service->update();
        $i = 1;
        if ($request['images']) {
            foreach ($request['images'] as $image) {

                $filename = $i . time() . '.' . $image->getClientOriginalExtension();
                $image->move(public_path('images/services'), $filename);

                $serviceImage = new ServiceImage();
                $serviceImage->service_id = $service->id;
                $serviceImage->image = $filename;
                $serviceImage->save();
                $i++;
            }

        }
        if ($request['deleted_images']) {
            $deleted_images = explode(',', $request['deleted_images']);
            ServiceImage::whereIn('id', $deleted_images)->delete();
        }


        return response()->json(['value' => true, 'data' => new ServicesResource($service)]);

    }

    public function deleteService($service_id)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }


        Service::destroy($service_id);

        return response()->json(['value' => true, 'data' => \Lang::get('data.DoneSuccess')]);


    }


    public function providerHome($flag, Request $request)
    {
        if ($request->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
            }

            if (is_numeric($flag)) {
                $services = Service::where('user_id', $user->id)->whereCategoryId($flag)->orderByDesc('id')->paginate(20);
            } else {
                $services = Service::where('user_id', $user->id)->orderByDesc('id')->paginate(20);
            }

        } else {
            if (is_numeric($flag)) {
                $services = Service::whereCategoryId($flag)->orderByDesc('id')->paginate(20);
            } else {
                $services = Service::orderByDesc('id')->paginate(20);
            }
        }
        return ServicesResource::collection($services)->additional([
            'value' => true
        ]);


    }

    public function providerSearch(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'search' => 'required'
        ]);
        if ($validator->passes()) {
            if ($request->header('Authorization')) {
                if (!$user = JWTAuth::parseToken()->authenticate()) {
                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
                }
                $services = Service::where('user_id', $user->id)->where('title', 'LIKE', "%{$request->search}%")->orderByDesc('id')->paginate(20);


            } else {
                $services = Service::where('title', 'LIKE', "%{$request->search}%")->orderByDesc('id')->paginate(20);

            }
            return ServicesResource::collection($services)->additional([
                'value' => true
            ]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function clientHome($flag, Request $request)
    {
        if (is_numeric($flag)) {
            $services = Service::whereCategoryId($flag)->orderByDesc('id')->paginate(20);
        } else {
            $services = Service::orderByDesc('id')->paginate(20);
        }
        return ServicesResource::collection($services)->additional([
            'value' => true
        ]);


    }

    public function clientSearch(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'search' => 'required'
        ]);
        if ($validator->passes()) {
            $services = Service::where('title', 'LIKE', "%{$request->search}%")->orderByDesc('id')->paginate(20);
            return ServicesResource::collection($services)->additional([
                'value' => true
            ]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }


    public function favouriteService($service_id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }
        $checkFav = Favourite::whereUserId($user->id)->whereServiceId($service_id)->first();
        if ($checkFav) {
            Favourite::whereUserId($user->id)->whereServiceId($service_id)->delete();
        } else {
            Favourite::create([
                'user_id' => $user->id,
                'service_id' => $service_id
            ]);
        }

        return response()->json(['value' => true, 'data' => \Lang::get('data.DoneSuccess')]);
    }

    public function favourites()
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }
        $rows = $user->favourites()->paginate(20);
        return ServicesResource::collection($rows)->additional([
            'value' => true
        ]);

    }
}
