<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\ChangeOldPasswordRequest;
use App\Http\Requests\Api\ChangePasswordRequest;
use App\Http\Requests\Api\CheckForgetCodeRequest;
use App\Http\Requests\Api\ForgetPassowrdRequest;
use App\Http\Requests\Api\LogoutRequest;
use App\Http\Requests\Api\UpdateProfileRequest;
use App\Http\Requests\Api\UserLoginRequest;
use App\Http\Requests\Api\UsersRequests;
use App\Http\Resources\Api\UsersResource;
use App\Models\Token;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\JWTException;
use JWTAuth;

class UsersController extends Controller
{
    public function register(UsersRequests $request)
    {
        $user = new User();
        $user->name = $request['name'];
        $user->mobile = $request['mobile'];
        $user->email = $request['email'] != "-1" ? $request['email'] : null;
        $user->city_id = $request['city_id'];
        $user->password = bcrypt($request['password']);
        $user->type = $request['type'];
        $user->category_id = $request['category_id'];

//        $user['verification_code'] = rand(1111, 9999);
        $user['verification_code'] = 1111;
        $user['status'] = 'not_active';
        $user['admin_accept'] = $request['type'] == 'client' ? 'true' : 'false';
        $user['verification_status'] = 'false';
        $user->save();
        $check = Token::where('token', $request['device_token'])->first();
        if ($check) {
            $check->status = 'connected';
            if ($request['device_type']) {
                $check->device_type = $request['device_type'];
            }
            $check['user_id'] = $user->id;
            $check->device_language = \App::getLocale();
            $check->update();
        } else {
            Token::create([
                'user_id' => $user->id,
                'token' => $request['device_token'],
                'device_type' => $request['device_type'],
                'status' => 'connected',
                'device_language' => \App::getLocale(),
            ]);
        }

        $data = new UsersResource($user);
        return response()->json(['value' => true, 'data' => $data]);


    }


    public function login(UserLoginRequest $request)
    {


        try {
            if (!$token = JWTAuth::attempt(['mobile' => $request->input('mobile'), 'password' => $request->input('password')], ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')]);
            }
        } catch (JWTException $e) {
            return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
        }
        $user = User::whereMobile($request['mobile'])->first();

        $check = Token::where('token', $request['device_token'])->first();
        if ($check) {
            $check->status = 'connected';
            if ($request['device_type']) {
                $check->device_type = $request['device_type'];
            }
            $check['user_id'] = $user->id;
            $check->device_language = \App::getLocale();
            $check->update();
        } else {
            Token::create([
                'user_id' => $user->id,
                'token' => $request['device_token'],
                'device_type' => $request['device_type'],
                'status' => 'connected',
                'device_language' => \App::getLocale(),
            ]);
        }

        $data = new UsersResource($user);
        return response()->json(['value' => true, 'data' => $data]);

    }


    public function check_forget_code(CheckForgetCodeRequest $request)
    {


        $user = User::whereMobile($request['mobile'])->first();
        if (!$user) return response()->json(['value' => true, 'msg' => \Lang::get('data.wrong_mobile')]);

        if ($user->verification_code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code')]);
        return response()->json(['value' => true, 'data' => \Lang::get('data.right_code')]);

    }


    public function forget_password(ForgetPassowrdRequest $request)
    {
        $user = User::whereMobile($request['mobile'])->first();
        if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile')]);
        $code = 1111;
//        $code = rand(1111, 9999);
        $user->verification_code = $code;
        $user->update();
        return response()->json(['value' => true, 'data' => $code]);
    }


    public function checkRegisterCode(CheckForgetCodeRequest $request)
    {
        $user = User::whereMobile($request['mobile'])->first();
        if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile')]);

        if ($user->verification_code != $request['code']) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code')]);


        } else {
            $user->verification_status = 'true';
            $user->status = 'active';
            $user->update();
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data]);

        }
    }


    public function change_password(ChangePasswordRequest $request)
    {


        $user = User::whereMobile($request['mobile'])->first();
        if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile')]);

        $lang = \App::getLocale();
        $user->password = bcrypt($request['password']);
        $user->update();
        $check = Token::where('token', $request['device_token'])->first();
        if ($check) {
            $check->status = 'connected';
            if ($request['device_type']) {
                $check->device_type = $request['device_type'];
            }
            $check['user_id'] = $user->id;
            $check->device_language = $lang;

            $check->update();
        } else {
            Token::create([
                'user_id' => $user->id,
                'token' => $request['device_token'],
                'device_type' => $request['device_type'],
                'status' => 'connected',
                'device_language' => $lang
            ]);
        }
        $data = new UsersResource($user);
        return response()->json(['value' => true, 'data' => $data]);

    }

    public function updateProfile(UpdateProfileRequest $request)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);

        }
        $user->name = $request->name ? $request['name'] : $user->name;
        $user->email = $request->email ? $request['email'] : $user->email;
        $user->mobile = $request->mobile ? $request['mobile'] : $user->mobile;
        $user->city_id = $request->city_id ? $request['city_id'] : $user->city_id;
        if ($request->hasFile('image')) {
            $file_name = time() . '.' . $request['image']->getClientOriginalExtension();
            $request['image']->move(public_path('images/users'), $file_name);
            $user->profile_image = $file_name;
        }


        $user->update();
        return response()->json(['value' => true, 'data' => new UsersResource($user)]);
    }

    public function changeOldPassword(ChangeOldPasswordRequest $request)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }

        if (\Hash::check($request['old_password'], $user->password)) {
            $user->password = bcrypt($request['new_password']);
            $user->update();
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match')]);
        }

    }

    public function userLogout(LogoutRequest $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);
        }


        $exist_token = Token::where(['token' => $request['device_token']])->first();
        if (!$exist_token) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token')]);

        $exist_token->status = 'disconnected';
        $exist_token->update();

        JWTAuth::invalidate(JWTAuth::getToken());

        return response()->json(['value' => true, 'data' => \Lang::get('data.logout')]);
    }
}
